// Copyright 1998-2019 Epic Games, Inc. All Rights Reserved.


#include "SocketClientCPPDemoGameModeBase.h"

void ASocketClientCPPDemoGameModeBase::BeginPlay() {
	//UE_LOG(LogTemp, Warning, TEXT("Begin Play"));
	isEndPlay = false;

	/** TCP **/
	//bind connection and message events
	USocketClientBPLibrary::getSocketClientTarget()->onsocketClientTCPConnectionEventDelegate.AddDynamic(this, &ASocketClientCPPDemoGameModeBase::socketClientTCPConnectionEvent);
	USocketClientBPLibrary::getSocketClientTarget()->onreceiveTCPMessageEventDelegate.AddDynamic(this, &ASocketClientCPPDemoGameModeBase::receiveTCPMessageEvent);

	//connect
	USocketClientBPLibrary::getSocketClientTarget()->connectSocketClientTCP("localhost", ESocketClientIPType::E_ipv4, 8888, EReceiveFilterClient::E_SAB, connectionIdGlobalTCP);


	/** UDP **/
	USocketClientBPLibrary::getSocketClientTarget()->onsocketClientUDPConnectionEventDelegate.AddDynamic(this, &ASocketClientCPPDemoGameModeBase::socketClientUDPConnectionEvent);
	USocketClientBPLibrary::getSocketClientTarget()->onreceiveUDPMessageEventDelegate.AddDynamic(this, &ASocketClientCPPDemoGameModeBase::receiveUDPMessageEvent);

	//start host
	USocketClientBPLibrary::getSocketClientTarget()->socketClientInitUDPReceiver(connectionIdGlobalUDP, "0.0.0.0", ESocketClientIPType::E_ipv4, 8888);
}

void ASocketClientCPPDemoGameModeBase::UnregisterAllComponents(bool bForReregister){
	Super::UnregisterAllComponents(bForReregister);

	if (!isEndPlay) {
		isEndPlay = true;
		//close my tcp connection
		USocketClientBPLibrary::getSocketClientTarget()->closeSocketClientConnectionTCP(connectionIdGlobalTCP);
		//stop my udp host
		USocketClientBPLibrary::getSocketClientTarget()->closeSocketClientConnectionUDP(connectionIdGlobalUDP);
	}
	
}

/** TCP **/


void ASocketClientCPPDemoGameModeBase::socketClientTCPConnectionEvent(const bool success, const FString message, const FString clientConnectionID)
{
	//output to screen
	if (success) {
		if (GEngine)
			GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Green, message);

		//send string message 
		FString stringMessage = "Hallo TCP";
		
		TArray<uint8> byteMessage;

		//Send a message after the connection has been established
		USocketClientBPLibrary::getSocketClientTarget()->socketClientSendTCP(clientConnectionID, stringMessage, byteMessage, true);
	}
	else {
		if (GEngine)
			GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Red, message);
	}

}


void ASocketClientCPPDemoGameModeBase::receiveTCPMessageEvent(const FString message, const TArray<uint8>& byteArray, const FString clientConnectionID)
{
	//output to screen
	if (GEngine)
		GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Blue, "TCP:"+message);
}


/** UDP **/


void ASocketClientCPPDemoGameModeBase::socketClientUDPConnectionEvent(const bool success, const FString message, const FString clientConnectionID)
{
	//output to screen
	if (success) {
		if (GEngine)
			GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Green, message);

		//send string message 
		FString stringMessage = "Hallo UDP";

		TArray<uint8> byteMessage;

		//Send a message after the hist is started
		USocketClientBPLibrary::getSocketClientTarget()->socketClientSendUDP("localhost", ESocketClientIPType::E_ipv4, 9999, stringMessage, byteMessage, false, connectionIdGlobalUDP);
	}
	else {
		if (GEngine)
			GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Red, message);
	}

}


void ASocketClientCPPDemoGameModeBase::receiveUDPMessageEvent(const FString message, const TArray<uint8>& byteArray, const  FString IP, const int32 port, const FString clientConnectionID)
{
	//output to screen
	if (GEngine)
		GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Blue, "UDP:" + message);
}